<?php
// +----------------------------------------------------------------------+
// | joPHPFlags.inc.php                                                   |
// |   the main include file.                                             |
// +----------------------------------------------------------------------+
// |        joPHPFlags project                                            |
// | coded in PHP version 4 in 2004 by Joel Thieffry & the zeRezo group   |
// +----------------------------------------------------------------------+
// |  This source file is subject to the Lesser GNU Public Licence, which |
// | text can be found at http://www.gnu.org/licenses/lgpl.txt .          |
// +----------------------------------------------------------------------+
// | Authors: Joel Thieffry <jo@zerezo.com>                               |
// +----------------------------------------------------------------------+
/*
 * 20040428 jo : initial version: 0.01.
 */

/////
// Some constants.
// ///

  /* An arbitrary default width/height ratio of a flag. */
  define('jPF_DEFAULT_RATIO', 3/2, true);
  /* An arbitrary default width of a flag. */
  define('jPF_DEFAULT_WIDTH', 324, true);
  
/////
// The flag data encapsulation class.
// ///

  /* The class to handle the flag data. */
  class joPHPFlags {

    /* The codename of the flag. ex: European Union, France, Red Cross. */
    var $codename;
    /* The width of the flag. */
    var $width;
    /* The height of the flag. */
    var $height;

    /*
     * Constructor.
     * @param _name The codename of the flag in the database.
     * @param _width The width of the image flag (0 if autoselect).
     * @param _height The height of the image flag (0 if autoselect).
     */
    function joPHPFlags($_name, $_width = 0, $_height = 0) {
      $this->codename = $_name;
      $this->width = $_width;
      $this->height = $_height;
    }

    /*
     * Get a new rendered image of the flag, with or without antialias.
     * @param _antialias True if rendering is antialiased (true if autoselect).
     * @return The rendered flag image.
     */
    function getImage($_antialias = true) {
      global $jPF_FlagDrawersDB;
      // Get the drawer function.
      $drawer = jPF_getFlagDrawer($this->codename);
      if (is_null($drawer))
        $drawer = 'jPF_FlagNotFound';
      // Get the ratio.
      $w_h_ratio = jPF_getFlagWHRatio($this->codename);
      if ($w_h_ratio == 0)
        $w_h_ratio = jPF_DEFAULT_RATIO;
      // Compute appropriate width/height.
      // TODO: need a lot more control over size.
      if ($this->width > 0) {
        if ($this->height == 0) {
          $this->height = $this->width / $w_h_ratio;
        }
      } else {
        if ($this->height > 0) {
          $this->width = $this->height * $w_h_ratio;
        } else {
          $this->width = jPF_DEFAULT_WIDTH;
          $this->height = $this->width / $w_h_ratio;
        }
      }
      // Render (with antialias or not) then return the result.
      if ($_antialias) {
        $imageSrc = $drawer($this->width*2, $this->height*2);
        $imageDest = ImageCreateTrueColor($this->width, $this->height);
        ImageCopyResampled($imageDest, $imageSrc, 0, 0, 0, 0, $this->width, $this->height, $this->width*2, $this->height*2);
      } else $imageDest = $drawer($this->width, $this->height);
      return $imageDest;
    }

  }

/////
// The flag drawers embedded database.
// ///

  /*
   * The Database of flag drawer function and ratio
   * (alphabetically sorted, codenames in lowercase).
   * array[0] = drawer function.
   * array[1] = w/h ratio (0 if unknown).
   */
  $jPF_FlagDrawersDB = array( // National flags.
                             'armenia' => array ('jPF_FlagArmenia', 2),
                             'austria' => array ('jPF_FlagAustria', 3/2),
                             'belgium' => array ('jPF_FlagBelgium', 15/13),
                             'bulgaria' => array ('jPF_FlagBulgaria', 3/2),
                             'czech republic' => array ('jPF_FlagCzech', 3/2),
                             'denmark' => array ('jPF_FlagDenmark', 37/28),
                             'england' => array ('jPF_FlagEngland', 5/3),
                             'estonia' => array ('jPF_FlagEstonia', 11/7),
                             'finland' => array ('jPF_FlagFinland', 18/11),
                             'france' => array ('jPF_FlagFrance', 3/2),
                             //'georgia' => array ('jPF_FlagGeorgia', 0), NEW FLAG IN 2004 !
                             'germany' => array ('jPF_FlagGermany', 5/3),
                             'greece' => array ('jPF_FlagGreece', 3/2),
                             'hungary' => array ('jPF_FlagHungary', 2),
                             'iceland' => array ('jPF_FlagIceland', 25/18),
                             'ireland' => array ('jPF_FlagIreland', 2),
                             'italy' => array ('jPF_FlagItaly', 3/2),
                             'japan' => array ('jPF_FlagJapan', 3/2),
                             'latvia' => array ('jPF_FlagLatvia', 2),
                             'lithuania' => array ('jPF_FlagLithuania', 2),
                             'luxembourg' => array ('jPF_FlagLuxembourg', 5/3),
                             'monaco' => array ('jPF_FlagMonaco', 5/4),
                             'netherlands' => array ('jPF_FlagNetherlands', 3/2),
                             'poland' => array ('jPF_FlagPoland', 4/3),
                             'romania' => array ('jPF_FlagRomania', 3/2),
                             'russia' => array ('jPF_FlagRussia', 3/2),
                             'scotland' => array ('jPF_FlagScotland', 3/2),
                             'spain' => array ('jPF_FlagSpain', 3/2),
                             'sweden' => array ('jPF_FlagSweden', 8/5),
                             'switzerland' => array ('jPF_FlagSwitzerland', 1),
                             'ukraine' => array ('jPF_FlagUkraine', 3/2),
                             'yugoslavia' => array ('jPF_FlagYugoslavia', 2),
                              // Miscellaneous flags.
                             'not found' => array ('jPF_FlagNotFound', 3/2),
                             'white' => array ('jPF_FlagWhite', 3/2),
                             );

  /*
   * Test if a flag exists.
   * ! NOT TESTED !
   * @param _name The codename of the flag to look for.
   * @return true if the flag is in database, otherwise false.
   */
  function jPF_isFlagInDB($_name = "") {
    global $jPF_FlagDrawersDB;
    $_name = strtolower($_name);
    if (isset($jPF_FlagDrawersDB[$_name]))
      return true;
    return false;
  }

  /*
   * Get the list of available flags.
   * @return An array.
   */
  function jPF_getFlagList() {
    global $jPF_FlagDrawersDB;
    return array_keys($jPF_FlagDrawersDB);
  }

  /*
   * Get the flag drawer, or null if not found.
   * @param _name The codename of the flag to look for.
   * @return The drawer function, otherwise null.
   */
  function jPF_getFlagDrawer($_name = "") {
    global $jPF_FlagDrawersDB;
    $drawer = null;
    $_name = strtolower($_name);
    $tab = @$jPF_FlagDrawersDB[$_name];
    if (! is_null($tab))
      $drawer = $tab[0];
    return $drawer;
  }

  /*
   * Get the flag w/h ratio, or 0 if not found.
   * @param _name The codename of the flag to look for.
   * @return The drawer function, otherwise null.
   */
  function jPF_getFlagWHRatio($_name = "") {
    global $jPF_FlagDrawersDB;
    $ratio = 0;
    $_name = strtolower($_name);
    $tab = @$jPF_FlagDrawersDB[$_name];
    if (! is_null($tab))
      $ratio = $tab[1];
    return $ratio;
  }

/////
// Image showers.
// ///

  /*
   * Send a generated image from flag image to the browser.
   * @param _image The flag image.
   * @param _format The format to generate the image in ("PNG", "GIF", "JPG" or "WBMP").
   */
  function showImage($_image, $_format) {
    if (is_null($_image))
      die("No image to generate !");
    $_format = strtoupper($_format);
    switch ($_format) {
      case 'PNG':
        if (!(imagetypes() & IMG_PNG))
          die("This server doesn't support PNG image format generation.");
        Header("Content-type: image/png");
        ImagePNG($_image);
        break;
      case 'JPG':
        if (!(imagetypes() & IMG_JPEG))
          die("This server doesn't support JPEG image format generation.");
        Header("Content-type: image/jpeg");
        ImageJPEG($_image);
        break;
      case 'WBMP':
        if (!(imagetypes() & IMG_WBMP))
          die("This server doesn't support WBMP image format generation.");
        Header("Content-type: Content-type: image/vnd.wap.wbmp");  // to check
        ImageWBMP($_image);
        break;
      default: // GIF is default
        if (!(imagetypes() & IMG_GIF))
          die("This server doesn't support GIF image format generation.");
        Header("Content-type: image/gif");
        ImageGIF($_image);
    }
  }

  /*
   * Send a generated PNG from flag image to the browser.
   * @param _image The flag image.
   */
  function showPNG($_image) { showImage($_image, 'PNG'); }

  /*
   * Send a generated GIF from flag image to the browser.
   * @param _image The flag image.
   */
  function showGIF($_image) { showImage($_image, 'GIF'); }

  /*
   * Send a generated JPG from flag image to the browser.
   * @param _image The flag image.
   */
  function showJPG($_image) { showImage($_image, 'JPG'); }

  /*
   * Send a generated WBMP from flag image to the browser.
   * @param _image The flag image.
   */
  function showWBMP($_image) { showImage($_image, 'WBMP'); }

/////
// Image transformations.
// ///

  /*
   * Add a simple black border to the image.
   * @param _image The image to be modified.
   */
  function imageAddBlackBorder($_image) {
    $black = ImageColorAllocate($_image, 0, 0, 0);
    $right = ImagesX($_image) - 1;
    $bottom = ImagesY($_image) - 1;
    imagerectangle($_image, 0, 0, $right, $bottom, $black);
  }

/////
// The flag drawers functions.
// ///

  /*
   * "Armenia" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagArmenia($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $blue = ImageColorAllocate($image, 0, 0, 153);
    $orange = ImageColorAllocate($image, 255, 153, 0);
    jPF_backgroundHorizontalStripes($image, array($red, $blue, $orange));
    return $image;
  }

  /*
   * "Austria" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagAustria($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $white = ImageColorAllocate($image, 255, 255, 255);
    jPF_backgroundHorizontalStripes($image, array($red, $white, $red));
    return $image;
  }

  /*
   * "Belgium" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagBelgium($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $black = ImageColorAllocate($image, 0, 0, 0);
    $yellow = ImageColorAllocate($image, 255 ,255 ,0);
    $red = ImageColorAllocate($image, 255, 0, 0);
    jPF_backgroundVerticalStripes($image, array($black, $yellow, $red));
    return $image;
  }

  /*
   * "Bulgaria" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagBulgaria($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $green = ImageColorAllocate($image, 0, 153, 0);
    $red = ImageColorAllocate($image, 255, 0, 0);
    jPF_backgroundHorizontalStripes($image, array($white, $green, $red));
    return $image;
  }

  /*
   * "Czech Republic" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagCzech($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $blue = ImageColorAllocate($image, 0, 0, 204);
    jPF_backgroundHorizontalStripes($image, array($white, $red));
    $midwidth = $_width/2;
    $midheight = $_height/2;
    ImageFilledPolygon($image, array(0, 0, $midwidth, $midheight, 0, $_height - 1), 3, $blue);
    return $image;
  }

  /*
   * "Denmark" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagDenmark($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $red = ImageColorAllocate($image, 188, 2, 4);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $right = $_width - 1;
    $bottom = $_height - 1;
    $unitW = $_width/37;
    $unitH = $_height/28;
    ImageFilledRectangle($image, 0, 0, $right, $bottom, $red);
    ImageFilledRectangle($image, 0, $unitH*12, $right, $unitH*16 - 1, $white);
    ImageFilledRectangle($image, $unitW*12, 0, $unitW*16 - 1, $bottom, $white);
    return $image;
  }

  /*
   * "England" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagEngland($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $right = $_width - 1;
    $bottom = $_height - 1;
    $unit = $_height/5;
    $center = $_width/2;
    ImageFilledRectangle($image, 0, 0, $right, $bottom, $white);
    ImageFilledRectangle($image, 0, $unit*2, $right, $unit*3 - 1, $red);
    ImageFilledRectangle($image, $center - $unit/2, 0, $center + $unit/2 - 1, $bottom, $red);
    return $image;
  }

  /*
   * "Estonia" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagEstonia($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $blue = ImageColorAllocate($image, 0, 102, 204);
    $black = ImageColorAllocate($image, 0, 0, 0);
    $white = ImageColorAllocate($image, 255, 255, 255);
    jPF_backgroundHorizontalStripes($image, array($blue, $black, $white));
    return $image;
  }

  /*
   * "France" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagFrance($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $blue = ImageColorAllocate($image, 0, 0, 102);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $red = ImageColorAllocate($image, 255, 0, 0);
    jPF_backgroundVerticalStripes($image, array($blue, $white, $red));
    return $image;
  }

  /*
   * "Finland" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagFinland($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $blue = ImageColorAllocate($image, 0, 51, 204);
    $right = $_width - 1;
    $bottom = $_height - 1;
    $unitW = $_width/18;
    $unitH = $_height/11;
    ImageFilledRectangle($image, 0, 0, $right, $bottom, $white);
    ImageFilledRectangle($image, 0, $unitH*4, $right, $unitH*7 - 1, $blue);
    ImageFilledRectangle($image, $unitW*5, 0, $unitW*8 - 1, $bottom, $blue);
    return $image;
  }

  /*
   * "Georgia" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagGeorgia($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $black = ImageColorAllocate($image, 0, 0, 0);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $violet = ImageColorAllocate($image, 128, 0, 64);
    $bwWidth = $_width*.4;
    $bwHeight = $_height/5;
    ImageFilledRectangle($image, 0, 0, $_width - 1, $_height - 1, $violet);
    ImageFilledRectangle($image, 0, 0, $bwWidth - 1, $bwHeight - 1, $black);
    ImageFilledRectangle($image, 0, $bwHeight, $bwWidth - 1, 2*$bwHeight - 1, $white);
    return $image;
  }
  /*
   * "Germany" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagGermany($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $black = ImageColorAllocate($image, 0, 0, 0);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $yellow = ImageColorAllocate($image, 255, 204, 0);
    jPF_backgroundHorizontalStripes($image, array($black, $red, $yellow));
    return $image;
  }

  /*
   * "Greece" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagGreece($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $blue = ImageColorAllocate($image, 0, 0, 153);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $crossSide = $_height/9;
    $squareSide = ($_height*5)/9;
    jPF_backgroundHorizontalStripes($image,
      array($blue, $white, $blue, $white, $blue, $white, $blue, $white, $blue));
    ImageFilledRectangle($image, 0, 0, $squareSide - 1, $squareSide - 1, $blue);
    ImageFilledRectangle($image, 0, 2*$crossSide, $squareSide - 1, 3*$crossSide - 1, $white);
    ImageFilledRectangle($image, 2*$crossSide, 0, 3*$crossSide - 1, $squareSide - 1, $white);
    return $image;
  }

  /*
   * "Hungary" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagHungary($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $green = ImageColorAllocate($image, 0, 132, 0);
    jPF_backgroundHorizontalStripes($image, array($red, $white, $green));
    return $image;
  }

  /*
   * "Iceland" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagIceland($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $blue = ImageColorAllocate($image, 0, 0, 204);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $right = $_width - 1;
    $bottom = $_height - 1;
    $unitW = $_width/25;
    $unitH = $_height/18;
    ImageFilledRectangle($image, 0, 0, $right, $bottom, $blue);
    ImageFilledRectangle($image, 0, $unitH*7, $right, $unitH*11 - 1, $white);
    ImageFilledRectangle($image, $unitW*7, 0, $unitW*11 - 1, $bottom, $white);
    ImageFilledRectangle($image, 0, $unitH*8, $right, $unitH*10 - 1, $red);
    ImageFilledRectangle($image, $unitW*8, 0, $unitW*10 - 1, $bottom, $red);
    return $image;
  }

/*
   * "Ireland" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagIreland($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $green = ImageColorAllocate($image, 0, 153, 0);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $orange = ImageColorAllocate($image, 255, 102, 0);
    jPF_backgroundVerticalStripes($image, array($green, $white, $orange));
    return $image;
  }

  /*
   * "Italy" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagItaly($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $green = ImageColorAllocate($image, 0, 153, 0);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $red = ImageColorAllocate($image, 255, 0, 0);
    jPF_backgroundVerticalStripes($image, array($green, $white, $red));
    return $image;
  }

/*
   * "Japan" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagJapan($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $right = $_width - 1;
    $bottom = $_height - 1;
    $radius = ($_height*3)/5;
    ImageFilledRectangle($image, 0, 0, $right, $bottom, $white);
    ImageFilledEllipse($image, $_width/2, $_height/2, $radius, $radius, $red);
    return $image;
  }

  /*
   * "Latvia" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagLatvia($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $red = ImageColorAllocate($image, 153, 0, 0);
    $white = ImageColorAllocate($image, 255, 255, 255);
    jPF_backgroundHorizontalStripes($image, array($red, $white, $red), array(2, 1, 2));
    return $image;
  }

  /*
   * "Lithuania" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagLithuania($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $yellow = ImageColorAllocate($image, 255, 255, 0);
    $green = ImageColorAllocate($image, 0, 153, 0);
    $red = ImageColorAllocate($image, 255, 0, 0);
    jPF_backgroundHorizontalStripes($image, array($yellow, $green, $red));
    return $image;
  }

  /*
   * "Luxembourg" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagLuxembourg($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $blue = ImageColorAllocate($image, 0, 153, 255);
    jPF_backgroundHorizontalStripes($image, array($red, $white, $blue));
    return $image;
  }

  /*
   * "Monaco" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagMonaco($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $white = ImageColorAllocate($image, 255, 255, 255);
    jPF_backgroundHorizontalStripes($image, array($red, $white));
    return $image;
  }

  /*
   * "Netherlands" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagNetherlands($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $red = ImageColorAllocate($image, 255, 0, 51);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $blue = ImageColorAllocate($image, 0, 51, 153);
    jPF_backgroundHorizontalStripes($image, array($red, $white, $blue));
    return $image;
  }

  /*
   * "Poland" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagPoland($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $red = ImageColorAllocate($image, 255, 0, 0);
    jPF_backgroundHorizontalStripes($image, array($white, $red));
    return $image;
  }

  /*
   * "Romania" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagRomania($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $blue = ImageColorAllocate($image, 0, 0, 204);
    $yellow = ImageColorAllocate($image, 255, 255, 0);
    $red = ImageColorAllocate($image, 255, 0, 0);
    jPF_backgroundVerticalStripes($image, array($blue, $yellow, $red));
    return $image;
  }

  /*
   * "Russia" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagRussia($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $blue = ImageColorAllocate($image, 0, 0, 255);
    $red = ImageColorAllocate($image, 255, 0, 0);
    jPF_backgroundHorizontalStripes($image, array($white, $blue, $red));
    return $image;
  }

  /*
   * "Scotland" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagScotland($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $blue = ImageColorAllocate($image, 0, 114, 198);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $right = $_width - 1;
    $bottom = $_height - 1;
    $unitW = $_width/9;
    $unitH = $_height/9;
    ImageFilledRectangle($image, 0, 0, $right, $bottom, $blue);
    ImageFilledPolygon($image, array (0, 0, $unitW, 0, $right, $bottom - $unitH, $right, $bottom, $right - $unitW, $bottom, 0, $unitH), 6, $white);
    ImageFilledPolygon($image, array (0, $bottom, 0, $bottom - $unitH, $right - $unitW, 0, $right, 0, $right, $unitH, $unitW, $bottom), 6, $white);
    return $image;
  }

  /*
   * "Spain" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagSpain($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $yellow = ImageColorAllocate($image, 255, 255, 0);
    jPF_backgroundHorizontalStripes($image, array($red, $yellow, $red), array(1, 2, 1));
    return $image;
  }

  /*
   * "Sweden" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagSweden($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $blue = ImageColorAllocate($image, 0, 102, 153);
    $yellow = ImageColorAllocate($image, 255, 204, 0);
    $right = $_width - 1;
    $bottom = $_height - 1;
    $unitW = $_width/16;
    $unitH = $_height/10;
    ImageFilledRectangle($image, 0, 0, $right, $bottom, $blue);
    ImageFilledRectangle($image, 0, $unitH*4, $right, $unitH*6 - 1, $yellow);
    ImageFilledRectangle($image, $unitW*5, 0, $unitW*7 - 1, $bottom, $yellow);
    return $image;
  }

  /*
   * "Switzerland" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagSwitzerland($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $white = ImageColorAllocate($image, 255, 255, 255);
    /*$crossWidth = $_width/7;
    $crossHeight = $_height/5.5;
    $crossHBorder = $_width*.27;
    $crossVBorder = $crossHeight;
    $squareWidth = ($_width - $crossWidth)/2;
    $squareHeight = ($_height - $crossHeight)/2;
    ImageFilledRectangle($image, 0, 0, $_width - 1, $_height - 1, $red);
    ImageFilledRectangle($image, $squareWidth, $crossVBorder, $squareWidth + $crossWidth - 1, $_height - $crossVBorder, $white);
    ImageFilledRectangle($image, $crossHBorder, $squareHeight, $_width - $crossHBorder, $squareHeight + $crossHeight - 1, $white);*/
    $centerX = $_width/2;
    $centerY = $_height/2;
    $spanCross = (min($_width, $_height)*2)/3; // The cross appear squared anywhere.
    $demiSpanCross = $spanCross/2;
    $demiWidthCross = ($spanCross*3)/20;
    ImageFilledRectangle($image, 0, 0, $_width - 1, $_height - 1, $red);
    ImageFilledRectangle($image, $centerX - $demiWidthCross, $centerY - $demiSpanCross, $centerX + $demiWidthCross - 1, $centerY + $demiSpanCross - 1, $white);
    ImageFilledRectangle($image, $centerX - $demiSpanCross, $centerY - $demiWidthCross, $centerX + $demiSpanCross - 1, $centerY + $demiWidthCross - 1, $white);
    return $image;
  }

  /*
   * "Ukraine" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagUkraine($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $blue = ImageColorAllocate($image, 0, 153, 255);
    $yellow = ImageColorAllocate($image, 255, 255, 0);
    jPF_backgroundHorizontalStripes($image, array($blue, $yellow));
    return $image;
  }

  /*
   * "Yugoslavia" flag drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   * @author jo@zerezo.com
   */
  function jPF_FlagYugoslavia($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $blue = ImageColorAllocate($image, 0, 0, 153);
    $white = ImageColorAllocate($image, 255, 255, 255);
    $red = ImageColorAllocate($image, 255, 0, 0);
    jPF_backgroundHorizontalStripes($image, array($blue, $white, $red));
    return $image;
  }

  /////
  // Miscellaneous Flags.
  // ///

  /*
   * Dummy "flag not found" drawer.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   */
  function jPF_FlagNotFound($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    /*$grey = ImageColorAllocate($image, 200, 200, 200);
    $blue = ImageColorAllocate($image, 0x5B, 0x69, 0xA6);
    $right = $_width-1;
    $bottom = $_height-1;
    ImageFilledRectangle($image, 0, 0, $right, $bottom, $grey);
    ImageFilledEllipse($image, $_width/2, $_height/2, $_width/3, $_height/3, $blue);*/
    $yellow = ImageColorAllocate($image, 255, 255, 0);
    $red = ImageColorAllocate($image, 255, 0, 0);
    $magenta = ImageColorAllocate($image, 255, 0, 255);
    $right = $_width-1;
    $bottom = $_height-1;
    $unitW = $_width/15;
    $unitH = $_height/10;
    ImageFilledRectangle($image, 0, 0, $right, $bottom, $yellow);
    ImageFilledRectangle($image, $unitW*5, $unitH, $unitW*7 - 1, $unitH*5 - 1, $magenta);
    ImageFilledRectangle($image, $unitW*8, $unitH, $unitW*10 - 1, $unitH*5 - 1, $magenta);
    ImageFilledEllipse($image, $unitW*6, $unitH*6.5, $unitW*2, $unitW*2, $magenta);
    ImageFilledEllipse($image, $unitW*9, $unitH*6.5, $unitW*2, $unitW*2, $magenta);
    $cornerString = "NOT FOUND";
    $cornerFont = 5;
    $cornerLeading = 4;
    $cornerRight = $right - (ImageFontWidth($cornerFont) * strlen($cornerString)) - $cornerLeading;
    $cornerBottom = $bottom - ImageFontHeight($cornerFont) - $cornerLeading;
    ImageString($image, $cornerFont, $cornerLeading, $cornerLeading, $cornerString, $red);
    ImageString($image, $cornerFont, $cornerRight, $cornerLeading, $cornerString, $red);
    ImageString($image, $cornerFont, $cornerLeading, $cornerBottom, $cornerString, $red);
    ImageString($image, $cornerFont, $cornerRight, $cornerBottom, $cornerString, $red);
    return $image;
  }

  /*
   * "white" flag drawer, for surrender.
   * @param _width The width of the image.
   * @param _height The height of the image.
   * @return The rendered image of the flag.
   */
  function jPF_FlagWhite($_width, $_height) {
    $image = ImageCreate($_width, $_height);
    $white = ImageColorAllocate($image, 255, 255, 255);
    ImageFilledRectangle($image, 0, 0, $_width-1, $_height-1, $white);
    return $image;
  }

/////
// The flag drawer helper functions.
// ///

  /*
   * Create the background of the image, with 3 big vertical stripes. ex: the French flag.
   * @param $_image The image to be modified.
   * @param $_colors An array of 3 colors, in order from 0 to 2.
   * @param $_proportions An array of proportions, in order from 0 to 2.
   */
  function jPF_background3VerticalStripes($_image, $_colors, $_proportions = array(1, 1, 1)) {
    $prop12 = $_proportions[0] + $_proportions[1];
    $sum = $prop12 + $_proportions[2];
    $width = ImagesX($_image);
    $right = $width - 1;
    $bottom = ImagesY($_image) - 1;
    ImageFilledRectangle($_image, 0, 0, $right, $bottom, $_colors[1]);
    ImageFilledRectangle($_image, 0, 0, ($_proportions[1]*$width)/$sum, $bottom, $_colors[0]);
    ImageFilledRectangle($_image, ($prop12*$width)/$sum, 0, $right, $bottom, $_colors[2]);
  }

  /*
   * Create the background of the image, with big vertical stripes. ex: France, Malta.
   * @param $_image The image to be modified.
   * @param $_colors An array of colors, in order from 0.
   * @param $_proportions An array of proportions, in order from 0.
   */
  function jPF_backgroundVerticalStripes($_image, $_colors, $_proportions = null) {
    $width = ImagesX($_image);
    $right = $width - 1;
    $bottom = ImagesY($_image) - 1;
    $nbColors = count($_colors);
    if (is_null($_proportions))
      $_proportions = array_fill(0, $nbColors, 1);
    $sumProp = array_sum($_proportions);
    $carriedProp = 0;
    $lastX = 0;
    for ($i = 0; $i < $nbColors; ++$i) {
      $carriedProp += $_proportions[$i];
      $newX = ($carriedProp*$width)/$sumProp - 1;
      ImageFilledRectangle($_image, $lastX, 0, $newX, $bottom, $_colors[$i]);
      $lastX = $newX + 1;
    }
  }

  /*
   * Create the background of the image, with big horizontal stripes. ex: Germany, Russia.
   * @param $_image The image to be modified.
   * @param $_colors An array of colors, in order from 0.
   * @param $_proportions An array of proportions, in order from 0.
   */
  function jPF_backgroundHorizontalStripes($_image, $_colors, $_proportions = null) {
    $right = ImagesX($_image) - 1;
    $height = ImagesY($_image);
    $bottom = $height - 1;
    $nbColors = count($_colors);
    if (is_null($_proportions))
      $_proportions = array_fill(0, $nbColors, 1);
    $sumProp = array_sum($_proportions);
    $carriedProp = 0;
    $lastY = 0;
    for ($i = 0; $i < $nbColors; ++$i) {
      $carriedProp += $_proportions[$i];
      $newY = ($carriedProp*$height)/$sumProp - 1;
      ImageFilledRectangle($_image, 0, $lastY, $right, $newY, $_colors[$i]);
      $lastY = $newY + 1;
    }
  }

?>